/****************************************************************************
** Meta object code from reading C++ file 'mltcontroller.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../src/mltcontroller.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'mltcontroller.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3Mlt16TransportControlE_t {};
} // unnamed namespace

template <> constexpr inline auto Mlt::TransportControl::qt_create_metaobjectdata<qt_meta_tag_ZN3Mlt16TransportControlE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Mlt::TransportControl",
        "play",
        "",
        "speed",
        "pause",
        "position",
        "stop",
        "seek",
        "rewind",
        "forceChangeDirection",
        "fastForward",
        "previous",
        "currentPosition",
        "next",
        "setIn",
        "setOut"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'play'
        QtMocHelpers::SlotData<void(double)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Slot 'play'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'pause'
        QtMocHelpers::SlotData<void(int)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 5 },
        }}),
        // Slot 'pause'
        QtMocHelpers::SlotData<void()>(4, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'stop'
        QtMocHelpers::SlotData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'seek'
        QtMocHelpers::SlotData<void(int)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 5 },
        }}),
        // Slot 'rewind'
        QtMocHelpers::SlotData<void(bool)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 9 },
        }}),
        // Slot 'fastForward'
        QtMocHelpers::SlotData<void(bool)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 9 },
        }}),
        // Slot 'previous'
        QtMocHelpers::SlotData<void(int)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 12 },
        }}),
        // Slot 'next'
        QtMocHelpers::SlotData<void(int)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 12 },
        }}),
        // Slot 'setIn'
        QtMocHelpers::SlotData<void(int)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Slot 'setOut'
        QtMocHelpers::SlotData<void(int)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<TransportControl, qt_meta_tag_ZN3Mlt16TransportControlE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Mlt::TransportControl::staticMetaObject = { {
    QMetaObject::SuperData::link<TransportControllable::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3Mlt16TransportControlE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3Mlt16TransportControlE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3Mlt16TransportControlE_t>.metaTypes,
    nullptr
} };

void Mlt::TransportControl::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<TransportControl *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->play((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 1: _t->play(); break;
        case 2: _t->pause((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 3: _t->pause(); break;
        case 4: _t->stop(); break;
        case 5: _t->seek((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 6: _t->rewind((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 7: _t->fastForward((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 8: _t->previous((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 9: _t->next((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 10: _t->setIn((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 11: _t->setOut((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *Mlt::TransportControl::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Mlt::TransportControl::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3Mlt16TransportControlE_t>.strings))
        return static_cast<void*>(this);
    return TransportControllable::qt_metacast(_clname);
}

int Mlt::TransportControl::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = TransportControllable::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 12)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 12;
    }
    return _id;
}
QT_WARNING_POP
